using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace PlanetTutorial
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Planet planet;
        public static Model sphere;

        Matrix Projection;
        Matrix View;
        
        Vector3 CameraPosition;

        PlanetType ptype = PlanetType.Ice;
        PlanetType currenttype = PlanetType.Undefined;
        GamePadState oldgps;
        int Seed = 0;
        SpriteFont font;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 1024;
            graphics.PreferredBackBufferHeight = 768;
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            CameraPosition = new Vector3(0, 0, -2);

            Projection = Matrix.CreateOrthographic(1024, 768, 1, 10000);
            View = Matrix.CreateLookAt(CameraPosition, Vector3.Zero, Vector3.Up);
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            sphere = Content.Load<Model>("sphere2");
            font = Content.Load<SpriteFont>("SpriteFont1");
            

            oldgps = GamePad.GetState(PlayerIndex.One);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            GamePadState gps = GamePad.GetState(PlayerIndex.One);
            if ((gps.DPad.Right == ButtonState.Pressed) && (oldgps.DPad.Right != ButtonState.Pressed))
            {
                ptype++;
                if (ptype > PlanetType.Molten)
                    ptype = PlanetType.Ice;

            }
            if ((gps.DPad.Left == ButtonState.Pressed) && (oldgps.DPad.Left != ButtonState.Pressed))
            {
                ptype--;
                if (ptype < PlanetType.Ice)
                    ptype = PlanetType.Molten;

            }
            if ((gps.DPad.Up == ButtonState.Pressed) && (oldgps.DPad.Up != ButtonState.Pressed))
            {
                Seed++;
                currenttype = PlanetType.Undefined;
            }
            if ((gps.DPad.Down == ButtonState.Pressed) && (oldgps.DPad.Down != ButtonState.Pressed))
            {
                Seed--;
                currenttype = PlanetType.Undefined;
            }
            if (planet != null)
            {
                if (gps.ThumbSticks.Right.Y < -0.5)
                {
                    planet.xOvercast -= 0.01f;
                    planet.Generate(GraphicsDevice);
                }
                if (gps.ThumbSticks.Right.Y > 0.5)
                {
                    planet.xOvercast += 0.01f;
                    planet.Generate(GraphicsDevice);
                }
            }
            oldgps = gps;
            if (currenttype != ptype)
            {
                switch (ptype)
                {
                    case PlanetType.Ice:
                        IcePlanet iplanet = new IcePlanet(GraphicsDevice,Seed);
                        iplanet.LoadContent(Content);
                        planet = iplanet;
                        break;

                    case PlanetType.Gas:
                        GasGiant gplanet = new GasGiant(GraphicsDevice, Seed);
                        gplanet.LoadContent(Content);
                        planet = gplanet;
                        break;

                    case PlanetType.Molten:
                        Molten mplanet = new Molten(GraphicsDevice, Seed);
                        mplanet.LoadContent(Content);
                        planet = mplanet;
                        break;

                    case PlanetType.Rock:
                        Rock rplanet = new Rock(GraphicsDevice, Seed);
                        rplanet.LoadContent(Content);
                        planet = rplanet;
                        break;

                    case PlanetType.Water:
                        WaterPlanet wplanet = new WaterPlanet(GraphicsDevice, Seed);
                        wplanet.LoadContent(Content);
                        planet = wplanet;
                        break;



                }
                planet.xOvercast = 1.1f;
                currenttype = ptype;
            }
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            GraphicsDevice.RenderState.DepthBufferEnable = false;
            GraphicsDevice.RenderState.CullMode = CullMode.None;
            planet.Draw(View,Matrix.CreateScale(200),Projection);

            if (planet != null)
            {
                spriteBatch.Begin();
                spriteBatch.DrawString(font, string.Format("xOvercast    {0}", planet.xOvercast), new Vector2(10, 10), Color.Black);
                spriteBatch.DrawString(font, string.Format("Seed         {0}", Seed), new Vector2(10, 30), Color.Black);
                spriteBatch.End();
            }
            base.Draw(gameTime);
            

        }
    }
}
