﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace PlanetTutorial
{
    public enum PlanetType
    {
        Ice,
        Water,
        Rock,
        Gas,
        Molten,
        Undefined
    };
    public abstract class Planet
    {
        public int Seed;
        public PlanetType Type;
        public Texture2D PermTex;
        public Texture2D Mercator;
        public Texture2D Palette;
        public Texture2D normalmap;
        public Texture2D lava;
        public Effect terrain;
        public Effect draw;
        public RenderTargetState rts;
        public double Nz;
        public float xOvercast = 1.1f;

        public void Draw(Matrix View, Matrix World, Matrix Projection)
        {
            Matrix wvp =  World * View * Projection;
            draw.Parameters["wvp"].SetValue(wvp);
            draw.Parameters["Palette"].SetValue(Palette);
            draw.Parameters["ColorMap"].SetValue(Mercator);
            draw.Parameters["BumpMap"].SetValue(normalmap);
            draw.Parameters["world"].SetValue(World);
            if (draw.Parameters["Lava"]!=null)
                draw.Parameters["Lava"].SetValue(lava);

           
            
            for (int pass = 0; pass < draw.CurrentTechnique.Passes.Count; pass++)
            {
                for (int msh = 0; msh < Game1.sphere.Meshes.Count; msh++)
                {
                    ModelMesh mesh = Game1.sphere.Meshes[msh];
                    for (int prt = 0; prt < mesh.MeshParts.Count; prt++)
                        mesh.MeshParts[prt].Effect = draw;
                    mesh.Draw();
                }
            }

        }

        public void Generate(GraphicsDevice graphics)
        {
            rts = new RenderTargetState(graphics, 512, 512, 512, 512);
            rts.BeginRenderToTexture();
            graphics.RenderState.DepthBufferEnable = false;
            graphics.RenderState.CullMode = CullMode.None;
            terrain.Parameters["ColorMap"].SetValue(PermTex);
            if (terrain.Parameters["xOvercast"] != null)
                terrain.Parameters["xOvercast"].SetValue(xOvercast);
            foreach (ModelMesh mesh in Game1.sphere.Meshes)
            {
                for (int i = 0; i < mesh.MeshParts.Count; i++)
                {
                    // Set this MeshParts effect (currentEffect) to the desired effect (from .fx file)   
                    mesh.MeshParts[i].Effect = terrain;
                }

                mesh.Draw();
            }
            Mercator = rts.EndRenderGetTexture();
            rts.DestroyBuffers();
            rts = null;

            Color[] Map = new Color[512 * 512];
            Mercator.GetData<Color>(Map);

            normalmap = new Texture2D(graphics, 512, 512, 1, TextureUsage.None, SurfaceFormat.Color);
            Color[] pixels = new Color[512 * 512];
            Color c3;


            for (int y = 0; y < 512; y++)
            {
                int offset = y * 512;
                for (int x = 0; x < 512; x++)
                {

                    float h0 = (float)Map[x + (512 * y)].R;
                    float h1 = (float)Map[x + (512 * safey(y + 1))].R;
                    float h2 = (float)Map[safex(x + 1) + (512 * y)].R;

                    float Nx = h0 - h2;
                    float Ny = h0 - h1;

                    Vector3 Normal = new Vector3((float)Nx, (float)Ny, (float)Nz);
                    Normal.Normalize();
                    Normal /= 2;

                    byte cr = (byte)(128 + (255 * Normal.X));
                    byte cg = (byte)(128 + (255 * Normal.Y));
                    byte cb = (byte)(128 + (255 * Normal.Z));
                    c3 = new Color(cr, cg, cb);
                    pixels[x + (y * 512)] = c3;
                }
            }

            normalmap.SetData<Color>(pixels);
        }
        int safex(int x)
        {
            if (x >= 512)
                return x - 512;
            if (x < 0)
                return 512 + x;
            return x;
        }

        int safey(int y)
        {
            if (y >= 512)
                return y - 512;
            if (y < 0)
                return 512 + y;
            return y;
        }
        public void BuildPerm(GraphicsDevice graf)
        {
            float[] map = new float[256];
            Random r = new Random(Seed);
            Vector4[] perm = new Vector4[256 * 256];
            int index = 0;
            for (int y = 0; y < 256; y++)
            {
                for (int i = 0; i < 256; i++)
                {
                    Vector4 a = new Vector4();
                    a.X = (float)((r.NextDouble()));
                    a.Y = (float)((r.NextDouble()));
                    a.Z = (float)((r.NextDouble()));
                    a.W = 1;
                    a.Normalize();
                    perm[index] = a;
                    index++;
                }
            }
            for (int i = 0; i < 256; i++)
            {
                map[i] = ((float)i) / 256.0f;
            }
            for (int i = 0; i < 256; i++)
            {
                int k = r.Next(255);
                float l = map[k];
                map[k] = map[i];
                map[i] = l;
            }
            for (int i = 0; i < 256; i++)
            {
                perm[i].W = map[i];
            }
            PermTex = new Texture2D(graf, 256, 256, 1, TextureUsage.None, SurfaceFormat.Vector4);
            PermTex.SetData<Vector4>(perm);
        }

    }
}
